# libraries
import json
from flask_mysqldb import MySQL
from flask_cors import CORS
from flask import Flask, request, jsonify

# Init application
app = Flask(__name__)
app.config.from_object(__name__)

# session variable
my_session = {'user_id': ''}

# connecting with database
CORS(app, resources={r"/*": {"origins": "*"}})
app.config['MYSQL_HOST'] = 'localhost'
app.config['MYSQL_USER'] = 'root'
app.config['MYSQL_PASSWORD'] = ''
app.config['MYSQL_DB'] = 'mediatrackerdb'

mysql = MySQL(app)

# sign in route


@app.route('/signin', methods=['POST', 'GET'])
def signin():
    if request.method == 'GET':
        # receive parameters
        username = request.args.get('username')
        password = request.args.get('password')
        if (username == '' or password == ''):
            return jsonify('EMPTY INPUTS')
        # init cursor
        cursor = mysql.connection.cursor()
        # execute query to check if user exists
        counter = cursor.execute(
            "SELECT (username) FROM user where username = %s", (username,))
        # if counter > 0, it means that user already exists, so it cannot be created again
        if (counter > 0):
            cursor.close()
            return jsonify('SIGNIN FAILURE')
        else:
            global my_session
            # If user doesn't exist, execute a query to insert into user table
            cursor.execute(
                "INSERT INTO user (username, password) VALUES (%s,%s)", (username, password))
            # commit changes in the database
            mysql.connection.commit()
            global my_session
            # select user's row
            cursor.execute(
                "SELECT * FROM user where username = %s and password = %s", (username, password))
            # fetch the attribute at position [0][0] (user's id) and store in the global variable to keep him logged in
            userid = cursor.fetchall()[0][0]
            my_session['user_id'] = userid
            # stop cursor
            cursor.close()
            # establish response
            return jsonify(userid)


# log in route
@app.route('/login', methods=['POST', 'GET'])
def login():
    if request.method == 'GET':
        # receive parameters
        username = request.args.get('username')
        password = request.args.get('password')

        # init cursor
        cursor = mysql.connection.cursor()
        # execute query to check if user exists
        counter = cursor.execute(
            "SELECT (username) FROM user where username = %s and password = %s", (username, password))

        # if counter > 0, it means that user already exists, so it can be logged in
        if (counter > 0):
            # init global variable to store the user's id
            global my_session
            # select user's row
            cursor.execute(
                "SELECT * FROM user where username = %s and password = %s", (username, password))
            # fetch the attribute at position [0][0] (user's id) and store in the global variable to keep him logged in
            userid = cursor.fetchall()[0][0]
            my_session['user_id'] = userid
            # close cursor
            cursor.close()
            # return userid, so it can be used in frontend
            return jsonify(userid)
        # if user doesn't exist just return a failure msg
        else:
            cursor.close()
            return jsonify('LOGIN FAILURE')


# log out route
@app.route('/logout', methods=['POST', 'GET'])
def logout():
    if request.method == 'GET':
        # check if there any user logged
        global my_session
        if (my_session['user_id'] != ''):
            # delete value from global variable and return success msg
            my_session['user_id'] = ''
            return jsonify('SUCCESS')
        else:
            return jsonify('FAILURE')


@app.route('/watched', methods=['POST', 'GET'])
def watched():
    if request.method == 'GET':
        # receive parameters
        item_id = request.args.get('item_id')
        item_type = request.args.get('item_type')
        item_state = request.args.get('item_state')
        user_id = request.args.get('user_id')
        item_title = request.args.get('item_title')
        item_cover = request.args.get('item_cover')

        # init cursor
        cursor = mysql.connection.cursor()

        # check if item_id exists
        counter = cursor.execute(
            "SELECT (item_id) FROM content where item_id = %s", (item_id,))

        # if counter > 0, it means that item_id already exists
        if (counter > 0):
            # check if item_state = watched
            checkState = cursor.execute(
                "SELECT (item_state) FROM content where item_id = %s and item_state=%s", (item_id, item_state))

            # if counter > 0, it means that item_state = watched, so it will be deleted from the collection
            if (checkState > 0):
                cursor.execute(
                    "DELETE FROM content WHERE item_id=%s", (item_id,))
                mysql.connection.commit()
                cursor.close()
                return jsonify('SUCCESSFULLY DELETED FROM YOUR COLLECTION')
            # if counter = 0, it means that item_state = plan to watch, so it will be modified to watched
            else:
                cursor.execute(
                    "UPDATE content SET item_state=%s WHERE item_id=%s", (item_state, item_id,))
                mysql.connection.commit()
                cursor.close()
                return jsonify('SUCCESSFULLY SET AS WATCHED')

        # if counter = 0, it means that item_id does not exist, so it will be added to the collection
        else:
            cursor.execute(
                "INSERT INTO content (item_id, item_type, item_state, user_id, item_cover, item_title) VALUES (%s,%s,%s,%s,%s,%s)", (item_id, item_type, item_state, user_id, item_cover, item_title))
            mysql.connection.commit()
            cursor.close()
            return jsonify('SUCCESSFULLY ADDED TO YOUR COLLECTION AS "WATCHED"')


@app.route('/plantowatch', methods=['POST', 'GET'])
def plantowatch():
    if request.method == 'GET':
        # receive parameters
        item_id = request.args.get('item_id')
        item_type = request.args.get('item_type')
        item_state = request.args.get('item_state')
        user_id = request.args.get('user_id')
        item_title = request.args.get('item_title')
        item_cover = request.args.get('item_cover')

        # init cursor
        cursor = mysql.connection.cursor()

        # check if item_id exists
        counter = cursor.execute(
            "SELECT (item_id) FROM content where item_id = %s", (item_id,))

        # if counter > 0, it means that item_id already exists
        if (counter > 0):
            # check if item_state = watched
            checkState = cursor.execute(
                "SELECT (item_state) FROM content where item_id = %s and item_state=%s", (item_id, item_state))

            # if counter > 0, it means that item_state = plan to watch, so it will be deleted from the collection
            if (checkState > 0):
                cursor.execute(
                    "DELETE FROM content WHERE item_id=%s", (item_id,))
                mysql.connection.commit()
                cursor.close()
                return jsonify('SUCCESSFULLY DELETED FROM YOUR COLLECTION')
            # if counter = 0, it means that item_state = watched, so it will be modified to plantowatch
            else:
                cursor.execute(
                    "UPDATE content SET item_state=%s WHERE item_id=%s", (item_state, item_id,))
                mysql.connection.commit()
                cursor.close()
                return jsonify('SUCCESSFULLY SET AS "PLAN TO WATCH"')

        # if counter = 0, it means that item_id does not exist, so it will be added to the collection
        else:
            cursor.execute(
                "INSERT INTO content (item_id, item_type, item_state, user_id, item_cover, item_title) VALUES (%s,%s,%s,%s,%s,%s)", (item_id, item_type, item_state, user_id, item_cover, item_title))
            mysql.connection.commit()
            cursor.close()
            return jsonify('SUCCESSFULLY ADDED TO YOUR COLLECTION AS "PLAN TO WATCH"')


@app.route('/getContent', methods=['POST', 'GET'])
def getContent():
    if request.method == 'GET':
        # receive parameters
        user_id = request.args.get('user_id')

        # init cursor
        cursor = mysql.connection.cursor()
        # select all rows that contain the user_id and fetch 'em
        counter = cursor.execute(
            "SELECT * FROM content where user_id = %s", (user_id,))
        items = cursor.fetchall()

        # create a dictionary, so it'll be easier to work with it
        insertItems = dict()
        for contador, resultado in enumerate(items):
            insertItems[contador] = {
                'id': resultado[0],
                'type': resultado[1],
                'state': resultado[2],
                'user_id': resultado[3],
                'image': resultado[4],
                'title': resultado[5]
            }
        # send data to front
        insertItemsJSON = json.dumps(insertItems)
        cursor.close()
        return insertItemsJSON


@app.route('/getItem', methods=['POST', 'GET'])
def getItem():
    if request.method == 'GET':
        # receive parameters
        id_list_string = request.args.get('id_list')
        global my_session
        user_id = my_session['user_id']
        print(id_list_string)

        id_list = list(id_list_string.split(","))
        itemStates = dict()

        cursor = mysql.connection.cursor()

        for contador, resultado in enumerate(id_list):
            counter = cursor.execute(
                "SELECT * FROM content where item_id = %s and user_id=%s", (id_list[contador], user_id))

            if counter > 0:
                state = cursor.fetchall()[0][2]
                # print(id + state)
                itemStates[contador] = {
                    'id': resultado,
                    'state': state
                }
        print(itemStates)
        print(jsonify(itemStates))
        return jsonify(itemStates)


    # If we are in our main file, the app runs
if __name__ == '__main__':
    app.run(host='localhost', port=5000, debug=True)
